@@ -1485,6 +1485,9 @@ int write_dict(PyObject* self, buffer_t buffer,
         Py_DECREF(value);
     }
     Py_DECREF(iter);
+    if (PyErr_Occurred()) {
+        return 0;
+    }
 
     /* write null byte and fill in length */
     if (!buffer_write_bytes(buffer, &zero, 1)) {
@@ -2245,27 +2248,31 @@ static PyObject* get_value(PyObject* self, const char* buffer,
          * Calling _error clears the error state, so fetch it first.
          */
         PyErr_Fetch(&etype, &evalue, &etrace);
-        InvalidBSON = _error("InvalidBSON");
-        if (InvalidBSON) {
-            if (!PyErr_GivenExceptionMatches(etype, InvalidBSON)) {
-                /*
-                 * Raise InvalidBSON(str(e)).
-                 */
-                Py_DECREF(etype);
-                etype = InvalidBSON;
 
-                if (evalue) {
-                    PyObject *msg = PyObject_Str(evalue);
-                    Py_DECREF(evalue);
-                    evalue = msg;
+        /* Dont reraise anything but PyExc_Exceptions as InvalidBSON. */
+        if (PyErr_GivenExceptionMatches(etype, PyExc_Exception)) {
+            InvalidBSON = _error("InvalidBSON");
+            if (InvalidBSON) {
+                if (!PyErr_GivenExceptionMatches(etype, InvalidBSON)) {
+                    /*
+                     * Raise InvalidBSON(str(e)).
+                     */
+                    Py_DECREF(etype);
+                    etype = InvalidBSON;
+
+                    if (evalue) {
+                        PyObject *msg = PyObject_Str(evalue);
+                        Py_DECREF(evalue);
+                        evalue = msg;
+                    }
+                    PyErr_NormalizeException(&etype, &evalue, &etrace);
+                } else {
+                    /*
+                     * The current exception matches InvalidBSON, so we don't
+                     * need this reference after all.
+                     */
+                    Py_DECREF(InvalidBSON);
                 }
-                PyErr_NormalizeException(&etype, &evalue, &etrace);
-            } else {
-                /*
-                 * The current exception matches InvalidBSON, so we don't need
-                 * this reference after all.
-                 */
-                Py_DECREF(InvalidBSON);
             }
         }
         /* Steals references to args. */
@@ -2313,17 +2320,19 @@ static PyObject* _elements_to_dict(PyObject* self, const char* string,
             PyObject *InvalidBSON;
 
             PyErr_Fetch(&etype, &evalue, &etrace);
-            InvalidBSON = _error("InvalidBSON");
-            if (InvalidBSON) {
-                Py_DECREF(etype);
-                etype = InvalidBSON;
+            if (PyErr_GivenExceptionMatches(etype, PyExc_Exception)) {
+                InvalidBSON = _error("InvalidBSON");
+                if (InvalidBSON) {
+                    Py_DECREF(etype);
+                    etype = InvalidBSON;
 
-                if (evalue) {
-                    PyObject *msg = PyObject_Str(evalue);
-                    Py_DECREF(evalue);
-                    evalue = msg;
+                    if (evalue) {
+                        PyObject *msg = PyObject_Str(evalue);
+                        Py_DECREF(evalue);
+                        evalue = msg;
+                    }
+                    PyErr_NormalizeException(&etype, &evalue, &etrace);
                 }
-                PyErr_NormalizeException(&etype, &evalue, &etrace);
             }
             PyErr_Restore(etype, evalue, etrace);
             Py_DECREF(dict);
