@@ -29,7 +29,9 @@ from pymongo.errors import (BulkWriteError,
                             InvalidOperation,
                             OperationFailure)
 from pymongo.message import (_INSERT, _UPDATE, _DELETE,
-                             _do_batched_write_command)
+                             _do_batched_write_command,
+                             _randint,
+                             _BulkWriteContext)
 from pymongo.write_concern import WriteConcern
 
 
@@ -123,7 +125,7 @@ def _merge_legacy(run, full_result, result, index):
             full_result['nUpserted'] += affected
         # Versions of MongoDB before 2.6 don't return the _id for an
         # upsert if _id is not an ObjectId.
-        elif result.get("updatedExisting") == False and affected == 1:
+        elif result.get("updatedExisting") is False and affected == 1:
             op = run.ops[index]
             # If _id is in both the update document *and* the query spec
             # the update document _id takes precedence.
@@ -181,11 +183,13 @@ def _merge_command(run, full_result, results):
         write_errors = result.get("writeErrors")
         if write_errors:
             for doc in write_errors:
+                # Leave the server response intact for APM.
+                replacement = doc.copy()
                 idx = doc["index"] + offset
-                doc["index"] = run.index(idx)
+                replacement["index"] = run.index(idx)
                 # Add the failed operation to the error document.
-                doc[_UOP] = run.ops[idx]
-            full_result["writeErrors"].extend(write_errors)
+                replacement[_UOP] = run.ops[idx]
+                full_result["writeErrors"].append(replacement)
 
         wc_error = result.get("writeConcernError")
         if wc_error:
@@ -276,15 +280,20 @@ class _Bulk(object):
             "nRemoved": 0,
             "upserted": [],
         }
+        op_id = _randint()
+        db_name = self.collection.database.name
+        listeners = self.collection.database.client._event_listeners
+
         for run in generator:
             cmd = SON([(_COMMANDS[run.op_type], self.collection.name),
                        ('ordered', self.ordered)])
             if write_concern.document:
                 cmd['writeConcern'] = write_concern.document
 
+            bwc = _BulkWriteContext(db_name, cmd, sock_info, op_id, listeners)
             results = _do_batched_write_command(
                 self.namespace, run.op_type, cmd,
-                run.ops, True, self.collection.codec_options, sock_info)
+                run.ops, True, self.collection.codec_options, bwc)
 
             _merge_command(run, full_result, results)
             # We're supposed to continue if errors are
@@ -306,6 +315,7 @@ class _Bulk(object):
         # If ordered is True we have to send GLE or use write
         # commands so we can abort on the first error.
         write_concern = WriteConcern(w=int(self.ordered))
+        op_id = _randint()
 
         for run in generator:
             try:
@@ -313,7 +323,8 @@ class _Bulk(object):
                     coll._insert(sock_info,
                                  run.ops,
                                  self.ordered,
-                                 write_concern=write_concern)
+                                 write_concern=write_concern,
+                                 op_id=op_id)
                 elif run.op_type == _UPDATE:
                     for operation in run.ops:
                         doc = operation['u']
@@ -326,13 +337,17 @@ class _Bulk(object):
                                      operation['upsert'],
                                      check_keys,
                                      operation['multi'],
-                                     write_concern=write_concern)
+                                     write_concern=write_concern,
+                                     op_id=op_id,
+                                     ordered=self.ordered)
                 else:
                     for operation in run.ops:
                         coll._delete(sock_info,
                                      operation['q'],
                                      not operation['limit'],
-                                     write_concern)
+                                     write_concern,
+                                     op_id,
+                                     self.ordered)
             except OperationFailure:
                 if self.ordered:
                     break
@@ -350,6 +365,7 @@ class _Bulk(object):
             "nRemoved": 0,
             "upserted": [],
         }
+        op_id = _randint()
         stop = False
         for run in generator:
             for idx, operation in enumerate(run.ops):
@@ -360,7 +376,9 @@ class _Bulk(object):
                     if run.op_type == _INSERT:
                         coll._insert(sock_info,
                                      operation,
-                                     write_concern=write_concern)
+                                     self.ordered,
+                                     write_concern=write_concern,
+                                     op_id=op_id)
                         result = {}
                     elif run.op_type == _UPDATE:
                         doc = operation['u']
@@ -373,12 +391,16 @@ class _Bulk(object):
                                               operation['upsert'],
                                               check_keys,
                                               operation['multi'],
-                                              write_concern=write_concern)
+                                              write_concern=write_concern,
+                                              op_id=op_id,
+                                              ordered=self.ordered)
                     else:
                         result = coll._delete(sock_info,
                                               operation['q'],
                                               not operation['limit'],
-                                              write_concern)
+                                              write_concern,
+                                              op_id,
+                                              self.ordered)
                     _merge_legacy(run, full_result, result, idx)
                 except DocumentTooLarge as exc:
                     # MongoDB 2.6 uses error code 2 for "too large".
@@ -567,6 +589,8 @@ class BulkOperationBuilder(object):
 
         :Parameters:
           - `document` (dict): the document to insert
+
+        .. seealso:: :ref:`writes-and-ids`
         """
         self.__bulk.add_insert(document)
 
