@@ -14,12 +14,14 @@
 
 """CommandCursor class to iterate over command results."""
 
+import datetime
+
 from collections import deque
 
 from bson.py3compat import integer_types
 from pymongo import helpers
-from pymongo.errors import AutoReconnect, CursorNotFound, NotMasterError
-from pymongo.message import _GetMore
+from pymongo.errors import AutoReconnect, NotMasterError, OperationFailure
+from pymongo.message import _CursorAddress, _GetMore, _convert_exception
 
 
 class CommandCursor(object):
@@ -50,8 +52,8 @@ class CommandCursor(object):
         """Closes this cursor.
         """
         if self.__id and not self.__killed:
-            self.__collection.database.client.close_cursor(self.__id,
-                                                           self.__address)
+            self.__collection.database.client.close_cursor(
+                self.__id, _CursorAddress(self.__address, self.__ns))
         self.__killed = True
 
     def close(self):
@@ -89,6 +91,8 @@ class CommandCursor(object):
         """Send a getmore message and handle the response.
         """
         client = self.__collection.database.client
+        listeners = client._event_listeners
+        publish = listeners.enabled_for_commands
         try:
             response = client._send_message_with_response(
                 operation, address=self.__address)
@@ -100,19 +104,53 @@ class CommandCursor(object):
             self.__killed = True
             raise
 
+        cmd_duration = response.duration
+        rqst_id = response.request_id
+        if publish:
+            start = datetime.datetime.now()
         try:
             doc = helpers._unpack_response(response.data,
                                            self.__id,
                                            self.__collection.codec_options)
-        except CursorNotFound:
+        except OperationFailure as exc:
             self.__killed = True
+
+            if publish:
+                duration = (datetime.datetime.now() - start) + cmd_duration
+                listeners.publish_command_failure(
+                    duration, exc.details, "getMore", rqst_id, self.__address)
+
             raise
-        except NotMasterError:
+        except NotMasterError as exc:
             # Don't send kill cursors to another server after a "not master"
             # error. It's completely pointless.
             self.__killed = True
+
+            if publish:
+                duration = (datetime.datetime.now() - start) + cmd_duration
+                listeners.publish_command_failure(
+                    duration, exc.details, "getMore", rqst_id, self.__address)
+
             client._reset_server_and_request_check(self.address)
             raise
+        except Exception as exc:
+            if publish:
+                duration = (datetime.datetime.now() - start) + cmd_duration
+                listeners.publish_command_failure(
+                    duration, _convert_exception(exc), "getMore", rqst_id,
+                    self.__address)
+            raise
+
+        if publish:
+            duration = (datetime.datetime.now() - start) + cmd_duration
+            # Must publish in getMore command response format.
+            res = {"cursor": {"id": doc["cursor_id"],
+                              "ns": self.__collection.full_name,
+                              "nextBatch": doc["data"]},
+                   "ok": 1}
+            listeners.publish_command_success(
+                duration, res, "getMore", rqst_id, self.__address)
+
         self.__id = doc["cursor_id"]
         if self.__id == 0:
             self.__killed = True
