@@ -456,10 +456,11 @@ class Database(common.BaseObject):
     def _list_collections(self, sock_info, slave_okay, criteria=None):
         """Internal listCollections helper."""
         criteria = criteria or {}
+        cmd = SON([("listCollections", 1), ("cursor", {})])
+        if criteria:
+            cmd["filter"] = criteria
+
         if sock_info.max_wire_version > 2:
-            cmd = SON([("listCollections", 1), ("cursor", {})])
-            if criteria:
-                cmd["filter"] = criteria
             coll = self["$cmd"]
             cursor = self._command(sock_info, cmd, slave_okay)["cursor"]
             return CommandCursor(coll, cursor, sock_info.address)
@@ -467,7 +468,8 @@ class Database(common.BaseObject):
             coll = self["system.namespaces"]
             res = _first_batch(sock_info, coll.full_name,
                                criteria, 0, slave_okay,
-                               CodecOptions(), ReadPreference.PRIMARY)
+                               CodecOptions(), ReadPreference.PRIMARY, cmd,
+                               self.client._event_listeners)
             data = res["data"]
             cursor = {
                 "id": res["cursor_id"],
@@ -485,15 +487,20 @@ class Database(common.BaseObject):
             will not include system collections (e.g ``system.indexes``)
         """
         with self.__client._socket_for_reads(
-            ReadPreference.PRIMARY) as (sock_info, slave_okay):
+                ReadPreference.PRIMARY) as (sock_info, slave_okay):
 
+            wire_version = sock_info.max_wire_version
             results = self._list_collections(sock_info, slave_okay)
-            names = [result["name"] for result in results]
-            if sock_info.max_wire_version <= 2:
-                # MongoDB 2.4 and older return index namespaces and collection
-                # namespaces prefixed with the database name.
-                names = [n[len(self.__name) + 1:] for n in names
-                         if n.startswith(self.__name + ".") and "$" not in n]
+
+        # Iterating the cursor to completion may require a socket for getmore.
+        # Ensure we do that outside the "with" block so we don't require more
+        # than one socket at a time.
+        names = [result["name"] for result in results]
+        if wire_version <= 2:
+            # MongoDB 2.4 and older return index namespaces and collection
+            # namespaces prefixed with the database name.
+            names = [n[len(self.__name) + 1:] for n in names
+                     if n.startswith(self.__name + ".") and "$" not in n]
 
         if not include_system_collections:
             names = [name for name in names if not name.startswith("system.")]
@@ -1037,6 +1044,9 @@ class Database(common.BaseObject):
             evaluated
           - `args` (optional): additional positional arguments are
             passed to the `code` being evaluated
+
+        .. warning:: the eval command is deprecated in MongoDB 3.0 and
+          will be removed in a future server version.
         """
         if not isinstance(code, Code):
             code = Code(code)
