@@ -915,6 +915,17 @@ class TestBulk(BulkTestBase):
         batch.execute()
         self.assertRaises(InvalidOperation, batch.execute)
 
+    def test_generator_insert(self):
+        def gen():
+            yield {'a': 1, 'b': 1}
+            yield {'a': 1, 'b': 2}
+            yield {'a': 2, 'b': 3}
+            yield {'a': 3, 'b': 5}
+            yield {'a': 5, 'b': 8}
+
+        result = self.coll.insert_many(gen())
+        self.assertEqual(5, len(result.inserted_ids))
+
 
 class TestBulkWriteConcern(BulkTestBase):
 
@@ -1032,7 +1043,7 @@ class TestBulkWriteConcern(BulkTestBase):
                   'op': {'_id': '...', 'a': 1}}]},
             result)
 
-        self.assertEqual(2, len(result['writeConcernErrors']))
+        self.assertTrue(len(result['writeConcernErrors']) > 1)
         failed = result['writeErrors'][0]
         self.assertTrue("duplicate" in failed['errmsg'])
 
